package com.yanzuoguang.util.thread;

import com.yanzuoguang.util.helper.StringHelper;
import com.yanzuoguang.util.log.Log;

/**
 * 线程执行类
 */
public class ThreadNext {

    public interface Next {
        /**
         * 执行下一个函数,出现异常会继续执行
         *
         * @return 是否继续执行
         */
        boolean next() throws Exception;

        /**
         * 沉睡时间
         *
         * @return
         */
        int getNextTime();
    }

    public static void start(Next next, String error) {
        if (next == null) {
            return;
        }
        new Thread(new Runnable() {
            @Override
            public void run() {
                boolean isExecute = true;
                Exception frontEx = null;
                while (isExecute) {
                    try {
                        isExecute = next.next();
                    } catch (Exception ex) {
                        if (frontEx == null || !(frontEx.getClass() == ex.getClass() &&
                                StringHelper.compare(frontEx.getMessage(), ex.getMessage()))) {
                            Log.error(ThreadNext.class, ex);
                        }
                        frontEx = ex;
                    }
                    if (isExecute) {
                        try {
                            int max = Math.max(next.getNextTime(), 100);
                            Thread.sleep(max);
                        } catch (InterruptedException e) {
                            e.printStackTrace();
                        }
                    } else {
                        break;
                    }
                }
            }
        }).start();
    }
}
