package com.yanzuoguang.util.vo;

import com.yanzuoguang.util.contants.ResultConstants;

/**
 * 接口之间的通讯结果
 */
public class ResponseResult<T> extends BaseVo {

    private static final long serialVersionUID = -508702000019378459L;

    /**
     * 请求状态码
     */
    private String code;

    /**
     * 服务消息
     */
    private String message;

    /**
     * 返回数据
     */
    private T data;

    /**
     * 数据源，用于显示异常数据
     */
    private Object target;

    /**
     * 获取结果状态码
     *
     * @return
     */
    public String getCode() {
        return code;
    }

    /**
     * 设置请求结果状态码
     *
     * @param code
     */
    public void setCode(String code) {
        this.code = code;
    }

    /**
     * 获取结果消息
     *
     * @return
     */
    public String getMessage() {
        return message;
    }

    /**
     * 设置结果消息
     *
     * @param message
     */
    public void setMessage(String message) {
        this.message = message;
    }

    /**
     * 获取数据
     *
     * @return
     */
    public T getData() {
        return data;
    }

    /**
     * 设置数据
     *
     * @param data
     */
    public void setData(T data) {
        this.data = data;
    }

    /**
     * 获取数据源
     *
     * @return 数据源
     */
    public Object getTarget() {
        return target;
    }

    /**
     * 设置数据源
     *
     * @param target
     */
    public void setTarget(Object target) {
        this.target = target;
    }

    /**
     * 构造函数
     */
    public ResponseResult() {
        this(ResultConstants.SUCCESS, "操作成功");
    }


    /**
     * 构造函数
     *
     * @param code    结果状态
     * @param message 结果消息
     */
    public ResponseResult(String code, String message) {
        this(code, message, null);
    }

    /**
     * 构造函数
     *
     * @param code    结果状态
     * @param message 结果熊希
     * @param data    结果
     */
    public ResponseResult(String code, String message, T data) {
        this(code, message, data, null);
    }

    /**
     * 构造函数
     *
     * @param code    结果状态
     * @param message 结果熊希
     * @param data    结果
     */
    public ResponseResult(String code, String message, T data, Object target) {
        this.code = code;
        this.message = message;
        this.data = data;
        this.target = target;
    }

    /**
     * 构造成功结果
     *
     * @param data 数据
     * @param <T>  数据类型
     * @return 一个请求成功的数据集合
     */
    public static final <T extends Object> ResponseResult<T> result(T data) {
        ResponseResult<T> ret = new ResponseResult<T>();
        ret.setCode(ResultConstants.SUCCESS);
        ret.setMessage("处理成功");
        ret.setData(data);
        return ret;
    }

    /**
     * 处理错误
     *
     * @param target 错误源对象
     * @return 处理的错误
     */
    public static final ResponseResult error(Object target) {
        return error(ResultConstants.UNKNOW_ERROR, "未知异常", target);
    }

    /**
     * 处理错误
     *
     * @param code    错误码
     * @param message 错误消息
     * @return 处理的错误
     */
    public static final ResponseResult error(String code, String message) {
        return error(code, message, null);
    }

    /**
     * 处理错误
     *
     * @param message 错误消息
     * @param target  错误标记
     * @return 处理的错误
     */
    public static final ResponseResult error(String message, Object target) {
        return error(ResultConstants.UNKNOW_ERROR, message, null);
    }

    /**
     * 处理错误
     *
     * @param code    错误码
     * @param message 错误消息
     * @param target  错误标记
     * @return 处理的错误
     */
    public static final ResponseResult error(String code, String message, Object target) {
        ResponseResult ret = new ResponseResult(code, message, null, target);
        return ret;
    }
}
