package com.yanzuoguang.dao.Impl;

import com.yanzuoguang.dao.DaoConst;
import com.yanzuoguang.dao.TableAnnotation;
import com.yanzuoguang.util.helper.StringHelper;
import com.yanzuoguang.util.base.MethodField;
import com.yanzuoguang.util.base.ObjectHelper;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 表结构的基本信息
 * created by yanzu on 2017/5/30.
 */
public class TableStruct {
    // 声明SQL语句模型
    private static final String insertModel = "INSERT INTO {TABLE}({FIELD}) VALUES({VALUES})";
    private static final String updateModel = "UPDATE {TABLE} SET {FIELD} WHERE 1=1 {WHERE}";
    private static final String removeModel = "DELETE FROM {TABLE} WHERE 1=1 {WHERE}";
    private static final String loadModel = "SELECT a.*{FIELD} FROM {TABLE} AS a {INNER} WHERE 1=1 {WHERE}";
    private static final String VERSON = "verson";
    private static final String ISREMOVE = "isremove";
    public static final String ISREMOVE_CONTANS = "contans____isremove";

    /**
     * 表名称
     */
    private String Name;

    /**
     * 主键名称
     */
    private TableFieldVo Key;

    /**
     * MD5KEY
     */
    private TableFieldVo MD5Key;

    /**
     * 其他字段
     */
    private List<TableFieldVo> Fields = new ArrayList<TableFieldVo>();

    public List<TableFieldVo> getFields() {
        return Fields;
    }

    public String getName() {
        return Name;
    }

    public void setName(String name) {
        Name = name;
    }

    public TableFieldVo getKey() {
        return Key;
    }

    public TableFieldVo getMD5Key() {
        return MD5Key;
    }

    public String getKeyName() {
        return this.Key.inputName;
    }

    public String getMD5KeyName() {
        return MD5Key != null ? MD5Key.inputName : null;
    }

    public Class<?> getKeyType() {
        return this.Key.type;
    }

    /**
     * 构造函数
     */
    public TableStruct() {
        this.Name = "";
        this.Key = new TableFieldVo("id");
    }

    /**
     * 通过实体的字段来创建表结构信息
     *
     * @param name 表名称
     * @param cls  关联的实体，主键放在第一位，其他字段放到后面；需要注意的是必需和表结构对应起来，会有隐性BUG，比如说在实体中增加了字段，会导致增加修改失败
     */
    public TableStruct(String name, Class<?> cls) {
        this.Name = name;

        // 获取实体中的所有字段信息，包含get、set、field
        HashMap<String, MethodField> fields = ObjectHelper.getTypeField(cls);

        // 遍历字段
        for (Map.Entry<String, MethodField> entry : fields.entrySet()) {

            // 字段信息获取
            MethodField field = entry.getValue();
            if (field.field == null && field.getMethod == null) {
                continue;
            }

            String fieldName = field.name;
            String fieldInputName = field.name;
            Class<?> fieldType = String.class;
            TableAnnotation annotation = null;
            if (field.field != null) {
                annotation = field.field.getAnnotation(TableAnnotation.class);
                fieldType = field.field.getType();
            } else if (field.getMethod != null) {
                annotation = field.getMethod.getAnnotation(TableAnnotation.class);
                fieldType = field.getMethod.getReturnType();
            }
            if (annotation != null) {
                fieldName = annotation.value();
            }

            TableFieldVo vo = new TableFieldVo(fieldName, fieldInputName, fieldType);
            // 判断是否属于统计字段
            if (vo.inputLName.equals("md5key")) {
                this.MD5Key = vo;
            }

            // 判断是否是主键，获取主键数据类型
            if (this.Key == null) {
                this.Key = vo;
            } else {
                this.Fields.add(vo);
            }
        }
    }

    public TableFieldVo getField(String name) {
        if (StringHelper.isEmpty(name)) {
            return null;
        }
        for (TableFieldVo fieldVo : this.Fields) {
            if (fieldVo.lName.equals(name.toLowerCase()) || fieldVo.inputLName.equals(name.toLowerCase())) {
                return fieldVo;
            }
        }
        return null;
    }

    /**
     * 判断是否包含版本号字段
     *
     * @return
     */
    public TableFieldVo getVersion() {
        return this.getField(VERSON);
    }

    /**
     * 判断是否包含remove字段
     *
     * @return
     */
    private TableFieldVo getRemove() {
        return this.getField(ISREMOVE);
    }

    /**
     * 通过表结构自动生成SQL语句
     *
     * @return
     */
    public void init(TableSqlCache table) {
        table.Table = this;
        if (!StringHelper.isEmpty(this.Name)) {
            table.add(releaseSqlCreate(), releaseSqlUpdate(), ReeleaseSqlRemove(), releaseSqlLoad());
        }
    }

    /**
     * 生成创建的SQL语句
     *
     * @return
     */
    private SqlData releaseSqlCreate() {
        // 生成添加的SQL语句
        String text = insertModel.replace("{TABLE}", this.Name);
        SqlData sql = new SqlData(DaoConst.Create, text);
        String flag = "";
        if (this.getKeyType() == String.class) {
            sql.addParaConst(this.Key.inputName, "{FIELD}", this.Key.name, "{VALUES}", "?");
            flag = ",";
        }
        for (TableFieldVo field : this.Fields) {
            sql.addParaConst(field.inputName, "{FIELD}", flag + field.name, "{VALUES}", flag + "?");
            flag = ",";
        }
        return sql;
    }

    /**
     * 生成修改的SQL语句
     *
     * @return
     */
    private SqlData releaseSqlUpdate() {
        // 生成添加的SQL语句
        String text = updateModel.replace("{TABLE}", this.Name);
        SqlData sql = new SqlData(DaoConst.Update, text);
        TableFieldVo removeField = this.getRemove();
        TableFieldVo versionField = this.getVersion();
        sql.addParaConst(this.Key.inputName,
                "{FIELD}", "" + this.Key.name + "=" + this.Key.name,
                "{WHERE}", " AND " + this.Key.name + "=?");
        for (TableFieldVo field : this.Fields) {
            if (field == removeField || field == versionField) {
                continue;
            }
            sql.addParaConst(field.inputName, "{FIELD}", "," + field.name + "=?");
        }
        if (removeField != null) {
            sql.addParaConst(removeField.inputName, "{WHERE}", " AND " + removeField.name + "=0");
        }
        if (versionField != null) {
            sql.addParaConst(versionField.inputName,
                    "{FIELD}", "," + versionField.name + "=1+" + versionField.name,
                    "{WHERE}", " AND " + versionField.name + "=?");
        }
        return sql;
    }

    /**
     * 生成删除的SQL语句
     *
     * @return
     */
    private SqlData ReeleaseSqlRemove() {
        TableFieldVo removeField = this.getRemove();
        TableFieldVo versionField = this.getVersion();
        if (removeField != null) {
            // 生成添加的SQL语句
            String text = updateModel.replace("{TABLE}", this.Name);
            SqlData sql = new SqlData(DaoConst.Remove, text);
            sql.addCode("{FIELD}", removeField.name + "=1");
            if (versionField != null) {
                sql.addCode("{FIELD}", "," + versionField.name + "=1+" + versionField.name);
            }
            addWhereField(sql, "");
            return sql;
        } else {
            String text = removeModel.replace("{TABLE}", this.Name);
            SqlData sql = new SqlData(DaoConst.Remove, text);
            addWhereField(sql, "");
            return sql;
        }
    }

    /**
     * 生成加载的SQL语句
     *
     * @return
     */
    private SqlData releaseSqlLoad() {
        // 生成添加的SQL语句
        String text = loadModel.replace("{TABLE}", this.Name);
        SqlData sql = new SqlData(DaoConst.Load, text);
        addWhereField(sql, "a.");
        return sql;
    }

    /**
     * 当前当前表所有字段的等于SQL语句
     *
     * @param sql
     */
    private void addWhereField(SqlData sql, String tag) {
        TableFieldVo removeField = this.getRemove();
        sql.add(this.Key.inputName, " AND " + tag + this.Key.name + "=?");
        for (TableFieldVo field : this.Fields) {
            sql.add(field.inputName, " AND " + tag + field.name + "=?");
        }
        if (removeField != null) {
            sql.addConst(ISREMOVE_CONTANS, " AND " + tag + removeField.name + "=0");
        }
    }

    /**
     * 生成统计的SQL语句
     *
     * @param sqlTableData 需要生成的实体
     * @param whereFields  WHERE字段
     * @param updateFields 需要增加的值的字段
     */
    public void addGroupSql(TableSqlCache sqlTableData, TableFieldString whereFields, TableFieldString updateFields) {
        sqlTableData.add(this.releaseSqlWhere(DaoConst.GroupQuery, loadModel, whereFields));
        sqlTableData.add(this.releaseSql(DaoConst.GroupAdd, updateModel, "{FIELD}={FIELD}+?", updateFields, new TableFieldString(this.Key.name)));
    }

    /**
     * 生成SQL语句
     *
     * @param name
     * @param model
     * @param whereFields
     * @return
     */
    public SqlData releaseSqlWhere(String name, String model, TableFieldString whereFields) {
        return this.releaseSql(name, model, "", new TableFieldString(), whereFields);
    }

    /**
     * 生成SQL语句
     *
     * @param name        SQL语句名称
     * @param model       SQL语句模板
     * @param valueModel  值字段模板
     * @param valueFields 值字段
     * @param whereFields WHERE字段
     * @return
     */
    public SqlData releaseSql(String name, String model, String valueModel, TableFieldString valueFields, TableFieldString whereFields) {
        // 参数字段
        List<String> paraFields = new ArrayList<String>();

        // 生成添加的SQL语句
        StringBuilder sbField = new StringBuilder();
        for (String field : valueFields.Fields) {
            if (sbField.length() > 0) {
                sbField.append(",");
            }
            sbField.append(valueModel.replace("{FIELD}", field));
            paraFields.add(field);
        }

        StringBuilder sbWhere = new StringBuilder();
        for (String field : whereFields.Fields) {
            sbWhere.append(" AND ");
            sbWhere.append(field);
            sbWhere.append("=?");
            paraFields.add(field);
        }

        String[] fields = new String[paraFields.size()];
        fields = paraFields.toArray(fields);
        // 生成修改的SQL语句
        String sql = model.replace("{TABLE}", this.Name).replace("{FIELD}", sbField.toString()).replace("{WHERE}", sbWhere.toString());
        SqlData ret = new SqlData(name, sql, fields);
        return ret;
    }

    /**
     * 生成根据某些字段不存在则保存的SQL语句
     *
     * @param tableStruct
     * @param sqlName
     * @param whereFields
     */
    public void addSaveWithSql(TableSqlCache tableStruct, String sqlName, TableFieldString whereFields) {
        tableStruct.add(this.releaseSqlWhere(sqlName, loadModel, whereFields));
    }


    /**
     * 生成判断数据是否存在的SQL语句
     *
     * @param sqlTableData
     * @param sqlName
     * @param fields
     */
    public void addExist(TableSqlCache sqlTableData, String sqlName, String[] fields) {
        String text = loadModel.replace("{TABLE}", this.Name);
        SqlData sql = new SqlData(sqlName, text);
        sql.addPara(this.Key.name, "{WHERE}", " AND a." + this.Key + "<>?");
        for (String field : fields) {
            sql.addParaConst(field, "{WHERE}", " AND a." + field + "=?");
        }
        if (getRemove() != null) {
            sql.addParaConst(ISREMOVE_CONTANS, "{WHERE}", " AND " + "a." + ISREMOVE + "=0");
        }
        sqlTableData.add(sql);
    }
}
