package com.yanzuoguang.db.Impl;

import com.yanzuoguang.db.DbExecute;
import com.yanzuoguang.extend.ConfigDb;
import com.yanzuoguang.util.vo.MapRow;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.support.rowset.SqlRowSet;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * 数据库操作类
 */
@Component
public class DbExecuteImpl implements DbExecute {

    @Resource
    @Qualifier("jdbcTemplate")
    private JdbcTemplate jdbc;

    @Resource
    private DbPrintSql printSql;

    @Resource
    private ConfigDb configDb;

    /**
     * 更新SQL语句的执行
     *
     * @param sql   SQL语句
     * @param paras 参数信息
     * @return
     */
    public int update(String sql, Object... paras) {
        int row = 0;
        try {
            sql = this.handleParas(sql, paras);
            row = jdbc.update(sql, paras);
            return row;
        } finally {
            printSql.print(row, sql, paras);
        }
    }

    /**
     * 查询数据，并返回集合
     *
     * @param cls   查询的结果的类型
     * @param sql   需要查询的SQL语句
     * @param paras 查询语句的参数
     * @param <T>   返回的集合的类型
     * @return 集合
     */
    public <T extends Object> List<T> query(Class<T> cls, String sql, Object... paras) {
        int row = 0;
        try {
            sql = this.handleParas(sql, paras);
            List<T> ret = jdbc.query(sql, paras, AllBeanRowMapper.getInstance(cls, configDb));
            if (ret == null) {
                ret = new ArrayList<T>();
            }
            row = ret.size();
            return ret;
        } finally {
            printSql.print(row, sql, paras);
        }
    }

    /**
     * 查询数据，并返回集合
     *
     * @param sql   需要查询的SQL语句
     * @param paras 查询语句的参数
     * @return 集合
     */
    public List<MapRow> query(String sql, Object... paras) {
        return query(MapRow.class, sql, paras);
    }

    /**
     * 查询第一个单元格的信息
     *
     * @param sql   SQL语句
     * @param paras 参数信息
     * @return 第一个单元格的数据
     */
    public Object queryCell(String sql, Object... paras) {
        int row = 0;
        try {
            sql = this.handleParas(sql, paras);
            SqlRowSet rowSet = jdbc.queryForRowSet(sql, paras);
            while (rowSet.next()) {
                row = 1;
                return rowSet.getObject(1);
            }
            return null;
        } finally {
            printSql.print(row, sql, paras);
        }
    }

    /**
     * 处理SQL语句和参数值
     *
     * @param sql
     * @param paras
     * @return
     */
    protected String handleParas(String sql, Object... paras) {
        return sql.replaceAll("1\\s*?=\\s*?1\\s*?(?i)AND", "")
                .replaceAll("(?i)WHERE\\s*?1\\s*?=\\s*?1", "");
    }

}
